<?php

namespace Drupal\graphql_compose\Plugin\GraphQL\DataProducer\Field;

use Drupal\Core\Cache\RefinableCacheableDependencyInterface;
use Drupal\file\FileInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;

/**
 * Returns an file style derivative of an file.
 *
 * @DataProducer(
 *   id = "field_file",
 *   name = @Translation("File"),
 *   description = @Translation("Returns file and derivatives."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("File properties")
 *   ),
 *   consumes = {
 *     "entity" = @ContextDefinition("entity",
 *       label = @Translation("Entity"),
 *       required = FALSE
 *     ),
 *   }
 * )
 */
class FieldFile extends DataProducerPluginBase {

  /**
   * {@inheritDoc}
   */
  public function resolve(FileInterface $entity = NULL, RefinableCacheableDependencyInterface $metadata) {
    if (!$entity) {
      return NULL;
    }

    $access = $entity->access('view', NULL, TRUE);
    $metadata->addCacheableDependency($access);
    if ($access->isAllowed()) {
      return [
        'url' => \Drupal::service('file_url_generator')->generateAbsoluteString($entity->getFileUri()),
        'name' => $entity->getFilename(),
        'size' => (int) $entity->getSize(),
        'mime' => $entity->getMimeType(),
      ];
    }

    return NULL;
  }

}
